% Wrapper for Indefinite one-class SVM
% 
% Input:           K: Training Kernel
%                  nu: one-class SVM parameter
%                rho: Kernel penalty parameter
%           accuracy: precision at which to stop Indefinite SVM
%           maxiters: maximum number of iterations
%           stepsize: stepsize for projected gradient algorithm if type=1
%               type: 1 for Projected Gradient Algorithm, 2 for ACCPM
%
% Output:      alpha: support vectors
%             primal: primal values recorded every "info" iterations
%               dual: dual values recorded every "info" iterations
%                gap: gap recorded every "info" iterations
%            CPUtime: CPUtime recorded every "info" iterations
%          itercount: iteration number recorded every "info" iterations
%
% Ronny Luss and Alexandre d'Aspremont, last modification: December 2008

function [alpha,primal,dual,gap,CPUtime,itercount] = Indefinite1classSVM(K,nu,rho,accuracy,maxiters,stepsize,type)
info=100;
n=size(K,1);

if nu==1 % there is only one possible solution
    disp('One-class SVM with nu==1 has the solution ones(n,1)/n by default.');
    alpha=ones(n,1)/n;primal=[];dual=[];gap=[];CPUtime=[];itercount=[];
    return;
end

x0 = projcutsimplex5(ones(n,1)/nu/n/2,ones(n,1),zeros(n,1),ones(n,1)/nu/n,1); % compute an initial point

switch type
    case{1} % Projected Gradient Algorithm
        [alpha,primal,dual,gap,CPUtime,itercount]=Indefinite1classSVM_gradient(K,nu,rho,accuracy,info,maxiters,x0,stepsize);

    case{2} % Analytic Center Cutting Plane Algorithm
        [alpha,primal,dual,gap,CPUtime,itercount]=Indefinite1classSVM_ACCPM(K,nu,rho,accuracy,info,maxiters,x0);
end
