% Wrapper for Indefinite SVR
% 
% Input:           K: Training Kernel
%             labels: Training labels
%                  C: SVR penalty parameter
%            eps_SVR: SVR epsilon tube parameter
%                rho: Kernel penalty parameter
%           accuracy: precision at which to stop Indefinite SVM
%           maxiters: maximum number of iterations
%           stepsize: stepsize for projected gradient algorithm if type=1
%               type: 1 for Projected Gradient Algorithm, 2 for ACCPM
%
% Output:      alpha: support vectors
%             primal: primal values recorded every "info" iterations
%               dual: dual values recorded every "info" iterations
%                gap: gap recorded every "info" iterations
%            CPUtime: CPUtime recorded every "info" iterations
%          itercount: iteration number recorded every "info" iterations
%
% Ronny Luss and Alexandre d'Aspremont, last modification: December 2008

function [alpha,primal,dual,gap,CPUtime,itercount] = IndefiniteSVR(K,labels,C,eps_SVR,rho,accuracy,maxiters,stepsize,type)
info=100;
x0 = projcutsimplex5(C*ones(length(labels),1)/2,ones(length(labels),1),-C*ones(length(labels),1),C*ones(length(labels),1),0); % compute an initial point

switch type
    case{1} % Projected Gradient Algorithm
        [alpha,primal,dual,gap,CPUtime,itercount]=IndefiniteSVR_gradient(K,labels,C,eps_SVR,rho,accuracy,info,maxiters,x0,stepsize);

    case{2} % Analytic Center Cutting Plane Algorithm
        [alpha,primal,dual,gap,CPUtime,itercount]=IndefiniteSVR_ACCPM(K,labels,C,eps_SVR,rho,accuracy,info,maxiters,x0);
end
